/***
 * Copyright (C) 1996-98. by Miroslav Zagorac <zaga@tirnanog.cc.fer.hr>
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <stdlib.h>

#include "../data/objects_def.h"
#include "../lib/lib_debug.h"

#include "cave.h"
#include "cave_data.h"
#include "graphics.h"
#include "level_editor.h"
#include "main.h"
#include "objects.h"


static short move_dir[] = { 1, CAVE_WIDTH, -1, -CAVE_WIDTH };

struct OBJECT_DATA *objects_data;
struct OBJECT_DATA  default_objects_data[] =
{
   {
      SPACE, SPACE, OBJECT_DESTROY_FLAG,
      {
         default_space,
         NULL
      }
   },
   {
      EXPLODE_S0, SPACE, 0,
      {
         default_explode_s0,
         default_explode_s1,
         default_explode_s1,
         default_explode_s0,
         default_explode_s0,
         default_explode_s2,
         NULL
      }
   },
   {
      EXPLODE_D0, DIAMOND, 0,
      {
         default_explode_d0,
         default_explode_d1,
         default_explode_d1,
         default_explode_d2,
         default_explode_d2,
         default_explode_d3,
         NULL
      }
   },
   {
      DIRT, DIRT, OBJECT_DESTROY_FLAG,
      {
         default_dirt,
         NULL
      }
   },
   {
      BRICK_WALL, BRICK_WALL, OBJECT_DROP_FLAG | OBJECT_DESTROY_FLAG,
      {
         default_brick_wall,
         NULL
      }
   },
   {
      MAGIC_WALL, MAGIC_WALL, OBJECT_DESTROY_FLAG,
      {
         default_magic_wall_0,
         default_magic_wall_0,
         default_magic_wall_1,
         default_magic_wall_1,
         default_magic_wall_2,
         default_magic_wall_2,
         default_magic_wall_3,
         default_magic_wall_3,
         NULL
      }
   },
   {
      WALL, WALL, OBJECT_DROP_FLAG,
      {
         default_steel_wall,
         NULL
      }
   },
   {
      DIAMOND, DIAMOND, OBJECT_FALL_FLAG | OBJECT_DROP_FLAG | OBJECT_DESTROY_FLAG,
      {
         default_diamond_0,
         default_diamond_1,
         default_diamond_2,
         default_diamond_3,
         default_diamond_4,
         default_diamond_5,
         default_diamond_6,
         default_diamond_7,
         NULL
      }
   },
   {
      DIAMOND_FALL, DIAMOND_FALL, OBJECT_FALL_FLAG | OBJECT_DROP_FLAG | OBJECT_DESTROY_FLAG,
      {
         default_diamond_0,
         default_diamond_1,
         default_diamond_2,
         default_diamond_3,
         default_diamond_4,
         default_diamond_5,
         default_diamond_6,
         default_diamond_7,
         NULL
      }
   },
   {
      FIREFLY_1, FIREFLY_1, OBJECT_DESTROY_FLAG,
      {
         default_firefly_0,
         default_firefly_0,
         default_firefly_1,
         default_firefly_1,
         default_firefly_2,
         default_firefly_2,
         default_firefly_3,
         default_firefly_3,
         NULL
      }
   },
   {
      BUTTERFLY_1, BUTTERFLY_1, OBJECT_DESTROY_FLAG,
      {
         default_butterfly_0,
         default_butterfly_0,
         default_butterfly_1,
         default_butterfly_2,
         default_butterfly_1,
         default_butterfly_0,
         default_butterfly_0,
         NULL
      }
   },
   {
      BOULDER, BOULDER, OBJECT_FALL_FLAG | OBJECT_DROP_FLAG | OBJECT_DESTROY_FLAG,
      {
         default_boulder,
         NULL
      }
   },
   {
      BOULDER_FALL, BOULDER_FALL, OBJECT_FALL_FLAG | OBJECT_DROP_FLAG | OBJECT_DESTROY_FLAG,
      {
         default_boulder,
         NULL
      }
   },
   {
      ROCKFORD, ROCKFORD, OBJECT_DESTROY_FLAG,
      {
         default_rockford_0,
         NULL
      }
   },
   {
      ROCKFORD_0, ROCKFORD_RANDOM, OBJECT_DESTROY_FLAG,
      {
         default_rockford_1,
         default_rockford_2,
         default_rockford_1,
         NULL
      }
   },
   {
      ROCKFORD_1, ROCKFORD_RANDOM, OBJECT_DESTROY_FLAG,
      {
         default_rockford_3,
         default_rockford_3,
         default_rockford_3,
         default_rockford_3,
         NULL
      }
   },
   {
      ROCKFORD_2, ROCKFORD_RANDOM, OBJECT_DESTROY_FLAG,
      {
         default_rockford_3,
         default_rockford_6, 
         default_rockford_6,
         default_rockford_3,
         default_rockford_3,
         default_rockford_6,
         default_rockford_6,
         default_rockford_3,
         default_rockford_3,
         default_rockford_6,
         default_rockford_6,
         default_rockford_3,
         NULL
      }
   },
   {
      ROCKFORD_3, ROCKFORD_RANDOM, OBJECT_DESTROY_FLAG,
      {
         default_rockford_3,
         default_rockford_3,
         default_rockford_4,
         default_rockford_5,
         default_rockford_4,
         default_rockford_3,
         default_rockford_3,
         NULL
      }
   },
   {
      ROCKFORD_R, ROCKFORD_R, OBJECT_DESTROY_FLAG,
      {
         default_rockford_r0,
         default_rockford_r1,
         default_rockford_r2,
         default_rockford_r3,
         default_rockford_r4,
         default_rockford_r5,
         default_rockford_r6,
         NULL
      }
   },
   {
      ROCKFORD_L, ROCKFORD_L, OBJECT_DESTROY_FLAG,
      {
         default_rockford_l0,
         default_rockford_l1,
         default_rockford_l2,
         default_rockford_l3,
         default_rockford_l4,
         default_rockford_l5,
         default_rockford_l6,
         NULL
      }
   },
   {
      PRE_OUTBOX, PRE_OUTBOX, OBJECT_DROP_FLAG | OBJECT_DESTROY_FLAG,
      {
         default_steel_wall,
         NULL
      }
   },
   {
      FLASH_OUTBOX, FLASH_OUTBOX, OBJECT_DESTROY_FLAG,
      {
         default_cave_out,
         default_cave_out,
         default_cave_out,
         default_cave_out,
         default_steel_wall,
         default_steel_wall,
         default_steel_wall,
         default_steel_wall,
         NULL
      }
   },
   {
      PRE_ROCKFORD_1, PRE_ROCKFORD_1, 0,
      {
         default_cave_out,
         NULL
      }
   },
   {
      PRE_ROCKFORD_2, ROCKFORD, 0,
      {
         default_steel_wall,
         default_steel_wall,
         default_steel_wall,
         default_steel_wall,
         default_cave_out,
         default_cave_out,
         default_cave_out,
         default_cave_out,
         default_steel_wall,
         default_steel_wall,
         default_steel_wall,
         default_steel_wall,
         default_cave_out,
         default_cave_out,
         default_cave_out,
         default_cave_out,
         default_steel_wall,
         default_steel_wall,
         default_steel_wall,
         default_steel_wall,
         default_cave_out,
         default_cave_out,
         default_cave_out,
         default_cave_out,
         default_explode_s2,
         default_explode_s2,
         default_explode_s0,
         default_explode_s0,
         default_explode_s1,
         default_explode_s1,
         NULL
      }
   },
   {
      AMOEBA, AMOEBA, OBJECT_DESTROY_FLAG,
      {
         default_amoeba_0,
         default_amoeba_0,
         default_amoeba_1,
         default_amoeba_1,
         default_amoeba_2,
         default_amoeba_2,
         default_amoeba_3,
         default_amoeba_3,
         NULL
      }
   },
   {
      WALL_PART, WALL_PART, 0,
      {
         default_wall_s0,
         default_wall_s1,
         default_wall_s2,
         default_wall_s3,
         default_wall_s4,
         default_wall_s5,
         default_wall_s6,
         default_wall_s7,
         NULL
      }
   },
   {
      PRE_MAGIC_WALL, PRE_MAGIC_WALL, OBJECT_DESTROY_FLAG,
      {
         default_brick_wall,
         NULL
      }
   },
   {
      POST_MAGIC_WALL, POST_MAGIC_WALL, OBJECT_DESTROY_FLAG,
      {
         default_brick_wall,
         NULL
      }
   },
   {
      UNKNOWN, UNKNOWN, 0,
      {
         unknown,
         NULL
      }
   }
};


#ifdef ANSI_FUNC

static inline short is_rockford (unchar object)
#else

static inline short is_rockford (object)
   unchar object;
#endif
{
   return ((object == ROCKFORD) || ((object >= ROCKFORD_0) && (object <= ROCKFORD_L)));
}


#ifdef ANSI_FUNC

static inline short is_firefly (unchar object)
#else

static inline short is_firefly (object)
   unchar object;
#endif
{
   return ((object >= FIREFLY_1) && (object <= FIREFLY_4));
}


#ifdef ANSI_FUNC

static inline short is_butterfly (unchar object)
#else

static inline short is_butterfly (object)
   unchar object;
#endif
{
   return ((object >= BUTTERFLY_1) && (object <= BUTTERFLY_4));
}


#ifdef ANSI_FUNC

static inline short is_magic_wall (unchar object)
#else

static inline short is_magic_wall (object)
   unchar object;
#endif
{
   return ((object == MAGIC_WALL) || (object == PRE_MAGIC_WALL) || (object == POST_MAGIC_WALL));
}


#ifdef ANSI_FUNC

unchar *get_object_image (unchar object, short position)
#else
   
unchar *get_object_image (object, position)
   unchar object;
   short position;
#endif
{
   short i;

   /***
    * Ova i slijedece funkcija se previse puta pozivaju u programu da bi se ovdje ista
    * ispisivalo u debug datoteku...
    */
   for (i = 0; objects_data[i].object_id != UNKNOWN; i++)
      if (object == objects_data[i].object_id)
         break;

   return (objects_data[i].image[(position < 0) ? -position : cave.anim[position]]);
}


#ifdef ANSI_FUNC

struct OBJECT_DATA *get_object (unchar object)
#else

struct OBJECT_DATA *get_object (unchar object)
   unchar object;
#endif
{
   short i;

   for (i = 0; objects_data[i].object_id != UNKNOWN; i++)
      if (object == objects_data[i].object_id)
         break;

   return (&objects_data[i]);
}


#ifdef ANSI_FUNC

static void clear_flags (unchar mask)
#else
   
static void clear_flags (unchar mask)
   unchar mask;
#endif
{
   short x, y;

   debug (("{%s(0x%02x)", __FUNCTION__, mask))

   for (y = 1; y < (cave_max.y - 1); y++)
      for (x = 1; x < (cave_max.x - 1); x++)
         cave.attr[CAVE_POSITION (x, y)] &= ~mask;

   debug_endfunc
}


#ifdef ANSI_FUNC

static inline short is_empty (short position)
#else

static inline short is_empty (position)
   short position;
#endif
{
   short x = position % CAVE_WIDTH, y = position / CAVE_WIDTH;

   if (!x || !y || (cave_max.x <= (x + 1)) || (cave_max.y <= (y + 1)))
      return (0);

   return ((cave.data[position] == SPACE) && !(cave.attr[position] & SCANNED_FLAG));
}


#ifdef ANSI_FUNC

static void mark_column (short position, short fall_direction)
#else

static void mark_column (position, fall_direction)
   short position;
   short fall_direction;
#endif
{
   debug (("{%s(%d, %d)", __FUNCTION__, position, fall_direction))

   cave.attr[position] |= fall_direction << 2;

   if (fall_direction == FALL_LEFT)
      position--;
   else if (fall_direction == FALL_RIGHT)
      position++;

   for ( ; is_empty (position); position += CAVE_WIDTH)
      cave.attr[position] |= SCANNED_FLAG;

   debug_endfunc
}


#ifdef ANSI_FUNC

void objects_fall (void)
#else

void objects_fall ()
#endif
{
   short x, y;

   debug (("{%s()", __FUNCTION__))

   for (y = 1; y < (cave_max.y - 1); y++)
      for (x = 1; x < (cave_max.x - 1); x++)
      {
         short position = CAVE_POSITION (x, y);

         /***
          * Moze li objekt pasti dolje?
          */
         if (! (cave.attr[position] & SCANNED_FLAG))
            if (get_object (cave.data[position])->flags & OBJECT_FALL_FLAG)
            {
               cave.attr[position] |= SCANNED_FLAG;

               if (is_empty (position + CAVE_WIDTH))
                  mark_column (position, FALL_DOWN);
               else if (get_object (cave.data[position + CAVE_WIDTH])->flags & OBJECT_DROP_FLAG)
	       {
                  if (is_empty (position - 1) && is_empty (position - 1 + CAVE_WIDTH))
                     mark_column (position, FALL_LEFT);
                  else if (is_empty (position + 1) && is_empty (position + 1 + CAVE_WIDTH))
                     mark_column (position, FALL_RIGHT);
	       }
            }
      }

   clear_flags (SCANNED_FLAG);

   for (y = 1; y < (cave_max.y - 1); y++)
      for (x = 1; x < (cave_max.x - 1); x++)
      {
         short position = CAVE_POSITION (x, y);

         if (! (cave.attr[position] & SCANNED_FLAG))
         {
            short fall_flag = 0, fall_direction = (cave.attr[position] & FALL_DIRECTION) >> 2;

            cave.attr[position] |= SCANNED_FLAG;

            if (fall_direction == FALL_DOWN)
            {
               if (cave.data[position + CAVE_WIDTH] == SPACE)
               {
                  cave_exchange_position (position + CAVE_WIDTH, position);

                  fall_flag = position += CAVE_WIDTH;
               }
            }
            else if (fall_direction == FALL_LEFT)
            {
               if (cave.data[position - 1] == SPACE)
               {
                  cave_exchange_position (position - 1, position);

                  position -= 1;
               }
            }
            else if (fall_direction == FALL_RIGHT)
            {
               if (cave.data[position + 1] == SPACE)
               {
                  cave_exchange_position (position + 1, position);

                  position += 1;
               }
            }

            /***
             * Ako dijamant ili kamen padaju prema dolje, onda se pretvaraju u isti tip
             * padajuceg objekta (i obratno).
             */
            if (fall_flag)
            {
               if (cave.data[position] == BOULDER)
                  cave.data[position] = BOULDER_FALL;
               else if (cave.data[position] == DIAMOND)
                  cave.data[position] = DIAMOND_FALL;
            }
            else if (cave.data[position] == BOULDER_FALL)
               cave.data[position] = BOULDER;
            else if (cave.data[position] == DIAMOND_FALL)
               cave.data[position] = DIAMOND;
         }
      }
   
   clear_flags (SCANNED_FLAG | FALL_DIRECTION);

   debug_endfunc
}


#ifdef ANSI_FUNC

void block_explode (unchar object, short position)
#else

void block_explode (object, position)
   unchar object;
   short position;
#endif
{
   static short fields[9] =
   {
      -1 - CAVE_WIDTH, -CAVE_WIDTH, 1 - CAVE_WIDTH,
                   -1,           0,              1,
      -1 + CAVE_WIDTH,  CAVE_WIDTH, 1 + CAVE_WIDTH
   };
   short i;

   debug (("{%s(%d, %d)", __FUNCTION__, object, position))

   for (i = 0; i < 9; i++)
   {
      short new_position = position + fields[i];
      short new_object = is_firefly (cave.data[new_position]) ? EXPLODE_S0 : (is_butterfly (cave.data[new_position]) ? EXPLODE_D0 : 0);

      debug ((" %02x:", cave.data[new_position]));

      if (get_object (cave.data[new_position])->flags & OBJECT_DESTROY_FLAG)
      {
         debug (("%d", i));

         if (is_rockford (cave.data[new_position]))
            game_data.game_status |= GAME_PLAYER_DEAD;

         cave_set_position (new_position, object, 0, 0);
      }
      
      if (new_object && fields[i])
         block_explode (new_object, new_position);
   }

   debug (("\n"))
   debug_endfunc
}


#ifdef ANSI_FUNC

void objects_explode (void)
#else

void objects_explode ()
#endif
{
   short x, y;

   debug (("{%s()", __FUNCTION__))

   for (y = 1; y < (cave_max.y - 1); y++)
      for (x = 1; x < (cave_max.x - 1); x++)
      {
         short i, position = CAVE_POSITION (x, y);

         /***
          * Ako neki padajuci tip objekta "stoji" na odgovajucem objektu, onda ide buuum.
          */
         if ((cave.data[position] == DIAMOND_FALL) || (cave.data[position] == BOULDER_FALL))
         {
            if (is_firefly (cave.data[position += CAVE_WIDTH]))
               block_explode (EXPLODE_S0, position);
            else if (is_butterfly (cave.data[position]))
               block_explode (EXPLODE_D0, position);
            else if (is_rockford (cave.data[position]))
               block_explode (EXPLODE_S0, position);
            else if (is_magic_wall (cave.data[position]))
            {
               game_data.game_status |= GAME_MAGIC_WALL;

               if ((cave.data[position + CAVE_WIDTH] == SPACE) && usec_to_sec (cave.wall_time))
                  cave_set_position (position + CAVE_WIDTH, (cave.data[position - CAVE_WIDTH] == BOULDER_FALL) ? DIAMOND : BOULDER, 0, 0);

               cave_set_position (position - CAVE_WIDTH, SPACE, 0, 0);
            }
         }
         else if (cave.data[position] == AMOEBA)
         {
            /***
             * Ovdje se testira da li je mozda firefly ili butterfly dodirnuo amoebu.
             */
            for (i = 0; i < 4; i++)
               if (is_butterfly (cave.data[position += move_dir[i]]))
                  block_explode (EXPLODE_D0, position + move_dir[i]);
               else if (is_firefly (cave.data[position + move_dir[i]]))
                  block_explode (EXPLODE_S0, position + move_dir[i]);
         }
         else if (is_firefly (cave.data[position]) || is_butterfly (cave.data[position]))
         {
            /***
             * Ovdje se testira da li je mozda firefly ili butterfly dodirnuo Rockforda.
             * Ako je, neizbjezno je ono sto slijedi...
             */
            for (i = 0; i < 4; i++)
               if (is_rockford (cave.data[position + move_dir[i]]))
               {
                  block_explode (is_firefly (cave.data[position]) ? EXPLODE_S0 : EXPLODE_D0, position);

                  break;
               }
         }
      }

   clear_flags (SCANNED_FLAG);

   debug_endfunc
}


#ifdef ANSI_FUNC

static void move_direction (short position, short direction)
#else
   
static void move_direction (position, direction)
   short position;
   short direction;
#endif
{
   short pos_1 = position + move_dir[(cave.attr[position] + direction) & MOVE_DIRECTION];
   short pos_2 = position + move_dir[cave.attr[position] & MOVE_DIRECTION];

   debug (("{%s(%d, %d)", __FUNCTION__, position, direction))

   /***
    * Objekt ce najprije probati skrenuti u njemu dragom pravcu, ako to ne ide onda ce
    * nastaviti ravno, a ako ni to ne ide, onda ce promijeniti smijer.
    */
   if (cave.data[pos_1] == SPACE)
   {
      cave_exchange_position (pos_1, position);

      cave.attr[pos_1] = (cave.attr[pos_1] & ~MOVE_DIRECTION) | ((cave.attr[pos_1] + direction) & MOVE_DIRECTION);
   }
   else if (cave.data[pos_2] == SPACE)
      cave_exchange_position (pos_2, position);
   else
      cave.attr[position] = (cave.attr[position] & ~MOVE_DIRECTION) | ((cave.attr[position] - direction) & MOVE_DIRECTION);

   debug_endfunc
}


#ifdef ANSI_FUNC

void objects_move (void)
#else

void objects_move ()
#endif
{
   short x, y, pos, amoeba_size = 0, amoeba_grow = 0;

   debug (("{%s()", __FUNCTION__))

   for (y = 1; y < (cave_max.y - 1); y++)
      for (x = 1; x < (cave_max.x - 1); x++)
      {
         pos = CAVE_POSITION (x, y);

         /***
          * Ako je vec netko sebi zauzeo mjesto za kretanje, onda nista...
          */
         if (cave.attr[pos] & SCANNED_FLAG)
            continue;

         cave.attr[pos] |= SCANNED_FLAG;

         /***
          * Firefly uvijek pokusava skrenuti u lijevo, a ako to ne ide onda nastavlja ravno.
          */
         if (is_firefly (cave.data[pos]))
            move_direction (pos, -1);
         /***
          * Za leptira vrijedi obrnuto, tj. prvo pokusava skrenuti u desno.
          */
         else if (is_butterfly (cave.data[pos]))
            move_direction (pos, 1);
         else if (cave.data[pos] == PRE_MAGIC_WALL)
         {
            if (game_data.game_status & GAME_MAGIC_WALL)
               cave_set_position (pos, MAGIC_WALL, 0, 0);
         }
         else if (cave.data[pos] == MAGIC_WALL)
         {
            if (! usec_to_sec (cave.wall_time))
               cave_set_position (pos, POST_MAGIC_WALL, 0, 0);
         }
         else if (cave.data[pos] == AMOEBA)
         {
            short i;

            amoeba_size++;

            /***
             * Moze li amoeba rasti i dalje?
             */
            for (i = 0; !amoeba_grow && (i < 4); i++)
               amoeba_grow += ((cave.data[pos + move_dir[i]] == DIRT) || (cave.data[pos + move_dir[i]] == DIRT)) ? 1 : 0;

            if ((random () % 100) < (usec_to_sec (cave.amoeba_time) ? 3 : 25))
            {
               short new_pos = pos + move_dir[random () % 3];

               if ((cave.data[new_pos] == SPACE) || (cave.data[new_pos] == DIRT))
                  cave_set_position (new_pos, AMOEBA, cave.attr[pos] | SCANNED_FLAG, cave.anim[pos]);
            }
         }
      }

   /***
    * Ako amoeba vise ne moze rasti onda se pretvara u dijamante, a ako je narasla previse,
    * onda u kamenje.
    */
   if (!amoeba_grow || (amoeba_size > game_data.max_amoeba_size))
      for (y = 1; y < (cave_max.y - 1); y++)
         for (x = 1; x < (cave_max.x - 1); x++)
         {
            pos = CAVE_POSITION (x, y);

            if (cave.data[pos] == AMOEBA)
               cave_set_position (pos, (amoeba_size > game_data.max_amoeba_size) ? BOULDER : DIAMOND, 0, 0);
         }

   clear_flags (SCANNED_FLAG);

   debug_endfunc
}


#ifdef ANSI_FUNC

void objects_animate (void)
#else

void objects_animate ()
#endif
{
   short x, y;

   debug (("{%s()", __FUNCTION__))

   for (y = 0; y < cave_max.y; y++)
   {
      if (game_data.game_status & GAME_BONUS_LIFE)
         object_space_change ();

      for (x = 0; x < cave_max.x; x++)
      {
         short               pos = CAVE_POSITION (x, y);
         struct OBJECT_DATA *object = get_object (cave.data[pos]);

         if (! object->image[++cave.anim[pos]])
         {
            cave.anim[pos] = 0;

            /***
             * Moze li se poceti igrati?
             */
            if (!(game_data.game_status & GAME_PLAY_MODE) && (cave.data[pos] == ROCKFORD))
            {
               game_data.game_status   |= GAME_PLAY_MODE;
               game_data.border_dist_x  = CAVE_BORDER_DIST_X;
               game_data.border_dist_y  = CAVE_BORDER_DIST_Y;
               game_data.scroll_step    = game_data.scroll_basic_step;
            }

            if ((object->object_id_next == ROCKFORD) || (object->object_id_next == ROCKFORD_RANDOM))
            {
               if ((random () % 100) < 5)
               {
                  static short rockford_random[] = { ROCKFORD_0, ROCKFORD_1, ROCKFORD_2, ROCKFORD_3 };

                  cave.data[pos] = rockford_random[random () & 0x03];
               }
               else
                  cave.data[pos] = ROCKFORD;
            }
            else
               cave.data[pos] = object->object_id_next;
         }
      }
   }

   game_data.game_status &= ~GAME_OBJECT_ANIMATE;

   debug_endfunc
}


#ifdef ANSI_FUNC

void object_space_change (void)
#else
   
void object_space_change ()
#endif
{
   static time_t  bonus_time;
   static ulong   old_game_status = 0;
   short          i, j;
   unchar        *space_image = get_object_image (SPACE, 0);

   debug (("{%s()", __FUNCTION__))

   if (!(old_game_status & GAME_BONUS_LIFE) && (game_data.game_status & GAME_BONUS_LIFE))
      bonus_time = time (NULL) + 2;

   if (bonus_time >= time (NULL))
   {
      for (j = 2; j < 32; j += 8)
         for (i = 0; i < 32; i += 2)
         {
            short color = (random () & 1) ? WHITE : BLACK;

            space_image[i + j * 32]           = (unchar) color;
            space_image[i + 1 + j * 32]       = (unchar) color;
            space_image[i + (j  + 1) * 32]    = (unchar) color;
            space_image[i + 1 + (j + 1) * 32] = (unchar) color;
         }
   }
   else
   {
      game_data.game_status &= ~GAME_BONUS_LIFE;

      for (i = 0; i < (32 * 32); i++)
         space_image[i] = BLACK;
   }

   old_game_status = game_data.game_status;

   debug_endfunc
}
